<?php
if (!defined('MEDIAWIKI'))
	die("MediaWiki extensions cannot be run directly.");
if (!defined('WikiDB')) {
	die("The WikiDB extension has not been set up correctly.<br>'"
		. basename(__FILE__) . "' should not be included directly.<br>Instead, "
		. "add the following to LocalSettings.php: "
		. "<tt><b>include_once(\"WikiDB/WikiDB.php\");</b></tt>");
}
/**
 * General-purpose utility functions for WikiDB.
 * These may be useful in other applications too.
 *
 * To avoid conflicts with other extensions (particularly my own) I either check
 * that the function hasn't already been defined prior to defining it, or else
 * I've prefixed the function name with "WikiDB_", to make the name unique.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2017, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 1376 $
 */

WikiDB_RegisterRevision('$Rev: 1376 $');

// WikiDB_IsNumeric()
// Returns true if the supplied value is a valid number.
// Should be used instead of PHP's is_numeric() function as it is a lot stricter.
// The PHP function will return true for strings like "23 Skidoo" or "", which we
// don't want!
// TODO: Should allow for internationalised numeric representations.
	function WikiDB_IsNumeric($Value) {
		if (preg_match('/^-?([0-9]+(\.[0-9]*)?|[0-9]*\.[0-9]+)$/', $Value))
			return true;
		else
			return false;
	}

// WikiDB_IsInteger()
// Same as IsNumeric() but does not allow non-integer values.  Note that
// "5.0" is currently not allowed, but "5" is.
// TODO: Should we allow non-integer representations of integers?
	function WikiDB_IsInteger($Value) {
		if (preg_match('/^-?[0-9]+$/', $Value))
			return true;
		else
			return false;
	}

// WikiDB_StrToNumber()
// Converts a string to a number, which is returned as a float().
// Zero is returned for non-numeric strings.
	function WikiDB_StrToNumber($Value) {
		if (WikiDB_IsNumeric($Value))
			return floatval($Value);
		else
			return floatval(0);
	}

// WikiDB_StrToInteger()
// Converts a string to an integer value. Zero is returned for non-numeric strings.
// Non-integer values are rounded to the nearest whole number.
// Note: Despite being an integer, we actually return a float value, so that we
//		 are not restricted to the platform's integer precision.
	function WikiDB_StrToInteger($Value) {
		return round(WikiDB_StrToNumber($Value));
	}

// WikiDB_FormatFloat()
// Formats a float value as a string, with trailing zeros removed and a minimum of
// 1 digit before the decimal point, according to the current user's language.
	function WikiDB_FormatFloat($Value) {
		global $wgLang;

		$Value = sprintf("%1f", $Value);
		$Value = preg_replace("/(\.\d+)0+$/U", "$1", $Value);

		return $wgLang->formatNum($Value);
	}

// WikiDB_FormatInteger()
// Formats an integer as a string, according to the user's language.
	function WikiDB_FormatInteger($Value) {
		global $wgLang;

		$Value = sprintf("%1.0f", $Value);

	// Temporary hack.  Currently 4-digit years are being detected as integers and
	// handled via this function, which results in years like "1976" being rendered
	// as "1,976", which is clearly wrong!
	// We therefore detect numbers with 4 digits or less, and skip the
	// language-specific formatting which introduces the thousands-grouping.
	// TODO: This is a temporary hack to avoid a current issue.  The correct way of
	//		 solving it is to add a Year data-type, so that anything explicitly typed
	//		 as Integer still gets the thousands-grouping for 4-digit numbers, but
	//		 with a priority so that 4-digit integers are treated as Year rather than
	//		 Integer for non-typed fields.
		if (strlen($Value) <= 4) {
			return $Value;
		}
		else {
			return $wgLang->formatNum($Value);
		}
	}

// WikiDB_NumberToSortableString()
// Takes a number and formats it into a string-representation suitable for sorting
// via a string sort.  If the input is not a valid number, then the function
// returns an empty string.
// TODO: Should be updated to allow for internationalised number formats to be
//		 passed in.
	function WikiDB_NumberToSortableString($Value) {
	// If value is blank, or is not a valid number, then return an empty string.
		if ($Value == "" || !WikiDB_IsNumeric($Value))
			return "";

	// Split the number on the decimal separator, to get the two components.
		$Parts = explode(".", $Value);

	// The part before the decimal point can simply be converted to an integer to
	// remove any leading zeros and to ensure that blank values are translated to
	// a zero (so that ".1" and "0.1" are considered equal).
		$Parts[0] = intval($Parts[0]);

	// The part after the decimal point needs to be trimmed of trailing zeros,
	// but any leading zeros need to be preserved, so we need to use a string
	// manipulation rather than using intval().
	// Also, if there is no second part, we normalise this to '0', so that
	// "1" and "1.0" are considered equal.
		if (isset($Parts[1])) {
			$Parts[1] = rtrim($Parts[1], "0");
			if (IsBlank($Parts[1]))
				$Parts[1] = "0";
		}
		else {
			$Parts[1] = 0;
		}

	// Set the sign byte.  We use two values that we know will be in the
	// correct sort order, regardless of the character set.
		if ($Parts[0] < 0) {
			$Parts[0] = abs($Parts[0]);
			$Sign = "n";
		}
		else
			$Sign = "p";

	// Field size is 255. 3 bytes are required for the initial sort index,
	// the sign and the decimal point. This leaves 126 bytes of padding for
	// each part. We use a space at the start, to ensure numbers are sorted
	// _before_ strings.  We don't bother to pad the numbers after the decimal
	// point, as they are left-aligned and so the padding makes no difference.
		return sprintf(" %s%126d.%s", $Sign, $Parts[0], $Parts[1]);
	}

// pWikiDB_ExpandVariables()
// Simple function to expand variables in a string, e.g. {{{2}}} is replaced with
// the value in element 2 of the array.  Used when we expand template arguments when
// rendering a <repeat> tag.
	function pWikiDB_ExpandVariables($Input, $Data, $Parser = null, $Frame = null) {
	// If $Parser and $Frame are present, we are on a version of MediaWiki that has
	// implemented a proper recursive parser, so we can do proper variable expansion
	// (including handling of defaults, parser functions, etc.)
		if (isset($Parser) && isset($Frame)) {
		// Fix to the way we handle fields which don't contain any data so that it is
		// possible for wiki editors to specify default values for blank fields.
		// Any blank fields are now removed from the data array, so when it comes to
		// template expansion it is if they don't exist, rather than being present
		// but blank, and therefore users can now specify default values that can be
		// used for missing/blank fields.
		// To stop this resulting in the field name being literally output in
		// situations where no default has been set (e.g. "{{{FieldName}}}" being
		// rendered as "{{{FieldName}}}") we replace any such instances with the
		// necessary markup to specify a blank default.  This means that
		// "{{{FieldName}}}" will be output as "".
		// TODO: Is this the best behaviour?  Should we treat these in the same way
		//		 as missing template arguments and leave it for the wiki editor to
		//		 ensure that proper defaults have been set up?
		// TODO: Should we handle missing (null) data differently from existing but
		//		 blank data?  I'm not sure we currently distinguish between the two,
		//		 but perhaps we should do?
			foreach ($Data as $Key => $Value) {
			// Replace any parameters which don't specify a default so that they
			// instead default to an empty string.
				$Key = preg_quote($Key, "/");
				$Input = preg_replace('/{{{\s*(' . $Key . ')\s*}}}/',
									  '{{{$1|}}}', $Input);

			// Remove any blank values from the data array so the MediaWiki parser
			// treats them as missing template parameters, and allows the specified
			// default.
				if (IsBlank($Value)) {
					unset($Data[$Key]);
				}
			// Otherwise, ensure the value is a string.  The MediaWiki pre-processor
			// trips up in the expand() function if this is not a string
			// representation.  For fields that have come from the wiki, this will
			// always be the case, but some fields may have come from meta-data and
			// may therefore be non-string representations (e.g. the '_Row' field is
			// an integer).
				else {
					$Data[$Key] = strval($Value);
				}
			}

		// Set up a new frame which mirrors the existing one but which also has the
		// field values as arguments.
		// If we are already in a template frame, merge the field arguments with the
		// existing template arguments first.
			if ($Frame instanceof PPTemplateFrame_DOM) {
				$NumberedArgs = $Frame->numberedArgs;
				$NamedArgs = array_merge($Frame->namedArgs, $Data);
			}
			else {
				$NumberedArgs = array();
				$NamedArgs = $Data;
			}
			$NewFrame = new PPTemplateFrame_DOM($Frame->preprocessor, $Frame,
												$NumberedArgs, $NamedArgs,
												$Frame->title);

		// Perform a recursive parse on the input, using our newly created frame.
			return $Parser->replaceVariables($Input, $NewFrame);
		}

	// If we get here, then we are unable to leverage the internal MediaWiki tools
	// to expand the variables, so we use our old very limited approach.
	// This is VERY crude, and may well fail in certain edge cases (or even, certain
	// 'non-edge' cases!)
		if (is_array($Data) && count($Data) > 0) {
		// Substitute variables.  Note that we need to replace $ with \$ within
		// $Value, to stop it being interpreted as a PHP variable reference.
			foreach ($Data as $Key => $Value) {
				$Input = preg_replace("/{{{" . preg_quote($Key, "/") . "}}}/",
									  str_replace('$', '\$', $Value),
									  $Input);
			}
		}
		return $Input;
	}

// WikiDB_Parse()
// Wrapper function to simplify text parsing.
// We use a local instance of the parser class, so that we don't get into a
// recursive nightmare (e.g. if a <repeat> tag contains a <repeat> tag).
// All tag hooks are ignored (as they are being dealt with already), however
// parser function hooks are parsed as expected.
	function WikiDB_Parse($Input, &$Parser, $LineStart = true, $Frame = null) {
		global $wgParser, $wgVersion;
		static $LocalParser;

	// If using MW >= 1.16 then we have a much cleaner way to deal with our
	// parsing (and one which works in all cases, hopefully).
		if (version_compare($wgVersion, "1.16", ">=")) {
			return $Parser->recursiveTagParse($Input, $Frame);
		}

	// If using MW < 1.16, then our old hacky version needs to be used.
	// This works reasonably well, but due to limitations in MediaWiki it will not
	// expand template variables when they are within WikiDB tags.
		else {
			if (!isset($LocalParser)) {
				$LocalParser = new Parser;

			// Later versions of MW (tested on 1.16) automatically add the necessary
			// parser function hooks to a new Parser instance, however earlier
			// versions (tested on 1.6) don't.  If they are not present, copy
			// them from the global $wgParser object.
				if (count($LocalParser->mFunctionHooks) == 0)
					$LocalParser->mFunctionHooks = $wgParser->mFunctionHooks;
			}

			$Output = $LocalParser->parse($Input, $Parser->mTitle, $Parser->mOptions,
										  $LineStart, true);
			return $Output->getText();
		}
	}

// WikiDB_IsValidTable()
// Wrapper for WikiDB_Table::IsValidTable().  That function can only be called once
// you've instantiated a table object, but often we need to check whether the raw
// title we have received refers to a valid table without having to create a table
// object for it.  This function allows us to do that.
// Arguments are the same as for the WikiDB_Table constructor, so are quite flexible.
// Note that previously we allowed the class method to be called both statically
// and non-statically, but this throws E_STRICT warnings, due to a stupid design
// choice by the PHP authors.  Oh well.  We write more complicated code to keep the
// error logs happy. :-(
	function WikiDB_IsValidTable($Title, $DefaultNamespace = null) {
		$Table = new WikiDB_Table($Title, $DefaultNamespace);
		return $Table->IsValidTable();
	}

// WikiDB_MakeTitle()
// $Title may either be a title object or a string representing the name of a
// page.  If $Title is a string, but doesn't include a namespace, then
// $DefaultNamespace is used if specified.  If no default is specified then
// WikiDB::GetDefaultTableNamespace() is used to fill in a sensible default.
	function WikiDB_MakeTitle($Title, $DefaultNamespace = null) {
	// If $Title is already a Title object, just return it.
	// TODO: Currently we assume that if it is an object, it is a Title object.
	//		 Is this sensible?
		if (is_object($Title)) {
			return $Title;
		}
	// Otherwise, it is a text string representing a page title.
	// In this case we create a new title object from that string, using the
	// default table namespace if necessary.
		else {
		// If no default namespace is specified, use the standard default.
			if ($DefaultNamespace === null)
				$DefaultNamespace = WikiDB::GetDefaultTableNamespace();

		// TODO: Should this use Title::newFromDBKey() instead?  If $Title contains
		//		 text it *should* always be a DB key, but it seems that the
		//		 transformations involved in newFromText() won't affect strings that
		//		 are already transformed, therefore it is safe (if marginally slower)
		//		 to pass it through there anyway - and that avoids problems if we
		//		 get passed in the actual title, rather than the DB key, for any
		//		 reason.
			return Title::newFromText($Title, $DefaultNamespace);
		}
	}

// WikiDB_SortTitles()
// Function to sort an array of titles.  This is required to ensure compatibility
// with MW < 1.13, when Title::compare() was added.
	function WikiDB_SortTitles(&$arrTitles) {
		if (method_exists("Title", "compare"))
			usort($arrTitles, array('Title', 'compare'));
		else
			usort($arrTitles, "pWikiDB_TitleSortCallback");
	}

// pWikiDB_TitleSortCallback()
// Duplicate of Title::compare() for versions of MediaWiki < 1.13, which is when it
// was added.
	function pWikiDB_TitleSortCallback($a, $b) {
		if ($a->getNamespace() == $b->getNamespace()) {
			return strcmp($a->getText(), $b->getText());
		}
		else {
			return $a->getNamespace() - $b->getNamespace();
		}
	}

// WikiDB_MakeLink()
// This function should be used by all internal code when generating links, to ensure
// it is fully portable across MW versions.
// The method for rendering a link has changed numerous times:
// * In 1.7 and below, Linker::makeLinkObj() needed to be called on an instantiated
//   object.  This was normally the user's Skin class, which is a child of Linker.
// * In 1.14, link() was added, which became the prefered method.  This has a
//   different calling signature (which has remained broadly unchanged in later
//	 incarnations).
// * In 1.18, the link() function was made static, so it should be called directly
//	 on the Linker class, rather than on an instantiated object.
// * In 1.27 the makeObjLink() function was removed altogether.
// * In 1.28, Linker::link() was deprecated in favour of the new LinkRenderer
//   class.
	function WikiDB_MakeLink($objTitle, $LinkText = null, $arrQueryParams = array())
	{
		global $wgUser;

	// Set up the arguments that will be passed to the selected function.  We do this
	// here so that the array can be modified for functions where the signature is
	// different.
		$arrArgs = array($objTitle, $LinkText, array(), $arrQueryParams);

	// If the MediaWikiServices class exists (it was introduced in MW 1.28)
	// then use this to get the link renderer object, which we can use to make the
	// link.
		if (class_exists("MediaWiki\\MediaWikiServices")) {
		// To ensure the code remains syntax-compatible with all supported versions
		// of PHP, we can't use a literal namespace.  Nor can we use
		// $ClassName::Method().
		// We therefore use a string variable and call_user_func().
			$ClassName = "MediaWiki\\MediaWikiServices";
			$MWServices = call_user_func(array($ClassName, "getInstance"));
			$LinkRenderer = $MWServices->getLinkRenderer();
			$Callable = array($LinkRenderer, "makeLink");
		}
	// Otherwise, we are using an older version of MediaWiki.
		else {
			$Skin = $wgUser->getSkin();

		// If the $Skin object has a 'link' function, we call it.  This is required
		// for MW versions between the function being implemented and before it being
		// rewritten to work statically (where we need to call it on an object
		// instance).
			if (method_exists($Skin, "link")) {
				$Callable = array($Skin, "link");
			}
		// Alternatively, this might be an older MW version, from before the function
		// was introduced.  In this case, makeLinkObj() should be used.
			elseif (method_exists($Skin, "makeLinkObj")) {
				$Callable = array($Skin, "makeLinkObj");

			// In this case the additional query arguments are passed as a string in
			// the third parameter, instead of an array in the fourth.
				$arrArgs[2] = wfArrayToCGI($arrArgs[3]);
				unset($arrArgs[3]);
			}
		// If none of the above apply, we are on a MW version between the point where
		// Linker::link() became a static function and where the LinkRenderer class
		// was introduced (in MW 1.28).  In this case, the static function is the one
		// to use.
			else {
				$Callable = array("Linker", "link");
			}
		}

		return call_user_func_array($Callable, $arrArgs);
	}

// GetCanonicalNamespaceName()
// Returns the canonical name of the specified namespace.  I've written a wrapper
// function, as this isn't as simple as I'd like!
	function WikiDB_GetCanonicalNamespaceName($NS) {
		global $wgCanonicalNamespaceNames;

	// If the namespace exists in $wgCanonicalNamespaceNames then get its canonical
	// name using the getCanonicalName() function of the MWNamespace class.
	// Originally, this class was called Namespace, but this became a reserved word
	// in PHP 5.3 and so in MW 1.13 it was renamed to MWNamespace.  Namespace
	// remained as an alias until it was finally removed altogether in MW 1.17.
		if (isset($wgCanonicalNamespaceNames[$NS])) {
		// In MW >= 1.13, the class we need is called MWNamespace, so if the
		// MWNamespace class exists, use it.
			if (class_exists("MWNamespace")) {
				$NSName = MWNamespace::getCanonicalName($NS);
			}
		// If MWNamespace is unavailable, then we are using a versions of MW < 1.13.
		// In these earlier versions, the class was called Namespace, so we use that
		// class instead.  However, we can't simply call this class in the normal
		// fashion, as it would cause parse errors if running on PHP >= 5.3,
		// therefore we need to use eval() to run the necessary code, as this won't
		// be parsed until it is actually executed.
			else {
				$PHPCode = 'return Namespace::getCanonicalName('
						 . intval($NS)
						 . ');';
				$NSName = eval($PHPCode);
			}
		}
	// If it is not defined, but $NS = 0, then it is the main namespace.  This
	// doesn't have an actual name, but we still need to report it.  We therefore
	// use 'main' for this.
		elseif ($NS == 0) {
			$NSName = WikiDB_Msg('blanknamespace');
		}
	// Otherwise, it is an undefined namespace.
		else {
			$NSName = WikiDB_Msg('undefined');
		}

		return $NSName;
	}

// WikiDB_GetLocalSpecialPageName()
// Cross-version wrapper for the MediaWiki code which gets the local name for a
// given special page.  This is the name that is displayed in the interface and also
// the name that is used for creating links to the page.
	function WikiDB_GetLocalSpecialPageName($CanonicalPageName) {
		global $wgContLang;

	// If SpecialPageFactory::getLocalNameFor() exists, then use this method as this
	// is the recommended method for modern versions of MediaWiki (available since
	// MW 1.18).
		if (is_callable(array('SpecialPageFactory', 'getLocalNameFor'))) {
			return SpecialPageFactory::getLocalNameFor($CanonicalPageName);
		}
	// Otherwise, use the old method, which gets the name via the Language class.
	// This was deprecated in MW 1.24 and removed altogether in MW 1.28.
		else {
			return $wgContLang->specialPage($CanonicalPageName);
		}
	}

// WikiDB_GetAction()
// Returns the current page action.  We wrap it into a function because the method
// for dealing with this has changed between MW versions.  This ensures we can
// easily maintain compatibility.
	function WikiDB_GetAction() {
		global $action, $mediaWiki;

	// The $mediaWiki object has existed since very early versions (was present in
	// MW 1.6, for example).  However the getAction() method wasn't added until
	// MW 1.18.  Therefore we need to check whether this function exists before
	// calling it, to ensure compatibility with older versions.
	// If it doesn't exist, we use the global $action variable instead, which is
	// the old way of getting this same information.
		if (is_callable(array($mediaWiki, "getAction")))
			return $mediaWiki->getAction();
		else
			return $action;
	}

// WikiDB_NewFromRedirect()
// Title::newFromRedirect was deprecated in MediaWiki 1.21, and removed altogether
// in MW 1.28, in favour of ContentHandler::makeContent().
// This function provides a wrapper that works in all supported versions of
// MediaWiki.
	function WikiDB_NewFromRedirect($PageText) {
		if (class_exists('ContentHandler')) {
			$objContentHandler = ContentHandler::makeContent($PageText, null,
															 CONTENT_MODEL_WIKITEXT);
			return $objContentHandler->getRedirectTarget();
		}
		else {
			return Title::newFromRedirect($PageText);
		}
	}

// WikiDB_Msg()
// Returns the text of the specified system message, using any additional arguments
// for variable replacement.
// Functionally equivalent to wfMsg(), which is the original way of handling this
// in MediaWiki.  However, this was deprecated in favour of wfMessage() and the new
// Message class in MW 1.17 and removed altogether in MW 1.27.
// This wrapper function calls wfMessage() if available, otherwise wfMsg(), and
// in both cases returns the expanded string (not a Message object).  As well as
// ensuring compatibility across MediaWiki versions, using this function simplifies
// our code as it doesn't need to explicitly call text() to get the output string.
// Note that the only difference between this and wfMsg() is that it benefits from
// some bug fixes that were applied to the wfMessage() implementation, namely that
// variable substitution now happens before template expansion, so constructs such
// as {{PLURAL:$1|egg|eggs}} now work as expected.
	function WikiDB_Msg($Key) {
		$Args = func_get_args();

		if (function_exists("wfMessage")) {
			$objMessage = call_user_func_array("wfMessage", $Args);
			return $objMessage->text();
		}
		else {
			return call_user_func_array("wfMsg", $Args);
		}
	}

// WikiDB_MsgExpanded()
// Same as WikiDB_Msg(), except it additionally parses the supplied key as wikitext
// and returns the resulting HTML.
// Equivalent to wfMsg($Text, array('parse')) and is required in order to retain
// support for earlier MW versions, now that wfMsgExt() has been removed.
	function WikiDB_MsgExpanded($Key) {
		global $wgOut;

	// In PHP 5.2, func_get_args() can't be used as a function parameter, so we need
	// to assign to a variable first.
		$Args = func_get_args();
		$Output = call_user_func_array("WikiDB_Msg", $Args);

		return $wgOut->parse($Output, true, true);
	}

// WikiDB_MsgExpandedInline()
// Same as WikiDB_MsgExpanded(), except it also removes the <p> tag that wraps the
// output, if present.
// Equivalent to wfMsg($Text, array('parseinline')) and is required in order to
// retain support for earlier MW versions, now that wfMsgExt() has been removed.
	function WikiDB_MsgExpandedInline($Key) {
		$Args = func_get_args();
		$Output = call_user_func_array("WikiDB_MsgExpanded", $Args);

	// Remove <p> tags.  This code was copied from wfMsgExt(), in MW 1.7.
		$arrMatches = array();
		if(preg_match( "~^<p>(.*)\n?</p>$~", $Output, $arrMatches)) {
			$Output = $arrMatches[1];
		}

		return $Output;
	}

// WikiDB_GetNamespaceIDs()
// Returns an array of namespace names => namespace IDs.  This is effectively a
// wrapper for $wgContLang->getNamespaceIds(), but in earlier versions of MediaWiki,
// this function doesn't exist, so in that situation we create the array ourselves
// using the same logic as is used in later MW versions.
// Required for MW 1.7 support, but possibly for later versions too as I'm not sure
// when the function was added.
	function WikiDB_GetNamespaceIDs() {
		global $wgContLang, $wgNamespaceAliases;
		static $NamespaceIDs;

		if (method_exists($wgContLang, "getNamespaceIds"))
			return $wgContLang->getNamespaceIds();
		else {
			if (is_null($NamespaceIDs)) {
			// Put namespace names and aliases into a hash table.
			// If this is too slow, then we should arrange it so that it is done
			// before caching. The catch is that at pre-cache time, the above
			// class-specific fix-up hasn't been done.
				$NamespaceIDs = array();
				foreach ($wgContLang->getNamespaces() as $Index => $Name) {
					$NamespaceIDs[$wgContLang->lc($Name)] = $Index;
				}
				if (method_exists($wgContLang, "getNamespaceAliases")) {
					foreach ($wgContLang->getNamespaceAliases() as $Name => $Index) {
						$NamespaceIDs[$wgContLang->lc($Name)] = $Index;
					}
				}
				if ($wgNamespaceAliases) {
					foreach ($wgNamespaceAliases as $Name => $Index) {
						$NamespaceIDs[$wgContLang->lc($Name)] = $Index;
					}
				}
			}

			return $NamespaceIDs;
		}
	}

// WikiDB_GetRevisionText()
// Revision::getText() was deprecated in MW 1.21 and removed altogether in MW 1.29.
// We therefore need to use a wrapper function so that we can perform this action on
// all supported MediaWiki versions.
	function WikiDB_GetRevisionText($Revision) {
		global $wgUser;

		if (method_exists($Revision, "getText")) {
			return $Revision->getText();
		}
		else {
			$objContent = $Revision->getContent(Revision::FOR_THIS_USER, $wgUser);
			return ContentHandler::getContentText($objContent);
		}
	}

// WikiDB_GetArticleText()
// Article::getContent() was deprecated in MW 1.21 and removed altogether in MW 1.29.
// We therefore need to use a wrapper function so that we can perform this action on
// all supported MediaWiki versions.
	function WikiDB_GetArticleText($Article) {
		global $wgUser;

		if (method_exists($Article, "getPage")) {
			$objWikiPage = $Article->getPage();
			$objContent = $objWikiPage->getContent(Revision::FOR_THIS_USER, $wgUser);
			return ContentHandler::getContentText($objContent);
		}
		else {
			return $Article->getContent();
		}
	}

// WikiDB_GetPaginationLinks()
// Cross-version wrapper for the MediaWiki prev/next link-generation code, as the
// location of this code will depend on the MediaWiki version being used.
	function WikiDB_GetPaginationLinks($Offset, $Limit, $Link, $QueryString = "",
									   $IsLastPage = false)
	{
	// From MediaWiki 1.19 onwards, the new location for this function is at
	// Language::viewPrevNext(), so use this if it exists.
		if (is_callable("Language::viewPrevNext")) {
			global $wgLang;

			$Query = wfCgiToArray($QueryString);

			if (is_object($Link)) {
				$Title = $Link;
			}
			else {
				$Title = Title::newFromText($Link);
				if (is_null($Title)) {
					return "";
				}
			}

			return $wgLang->viewPrevNext($Title, $Offset, $Limit, $Query,
										 $IsLastPage);
		}
	// Otherwise, use the old, global function.  This was deprecated in MediaWiki
	// 1.19, and removed altogether in MediaWiki 1.24.
		else {
			return wfViewPrevNext($Offset, $Limit, $Link, $QueryString, $IsLastPage);
		}
	}

// WikiDB_WrapDBResult()
// Prior to MediaWiki 1.11 various DB functions (e.g. query() or select()) returned
// a raw result object from the DB back-end (e.g. a "mysql result" resource).
// For compatibility with older versions, we therefore need to convert this into
// a ResultWrapper object if it isn't one already.
// Luckily, the ResultWrapper class is present in all versions back to at least
// 1.7 (the lowest we support), so this is safe.  It just wasn't used properly
// until 1.11, for some reason.
// This function should be used to wrap DB results to ensure compatibility with all
// supported MediaWiki versions.
	function WikiDB_WrapDBResult($DB, $Result) {
		$ClassName = "ResultWrapper";
		if (!($Result instanceof $ClassName)) {
			$Result = $DB->resultObject($Result);
		}

		return $Result;
	}

// WikiDB_GetDBType()
// Wrapper for Database::getType(), which was added in MediaWiki 1.16.  Earlier
// versions need to pull it from $wgDBType, directly.
	function WikiDB_GetDBType($DB) {
		global $wgDBtype;

		if (method_exists($DB, "getType")) {
			return $DB->getType();
		}
		else {
			return strtolower(trim($wgDBtype));
		}
	}

// IsBlank()
// A staggeringly useful function when you don't care much about the type of blank.
// We wrap in a function_exists() check so that it plays nicely when used with my
// other extensions, some of which also include this function.
if (!function_exists("IsBlank")) {
	function IsBlank($Input) {
	// is_null() returns true if $Input is unset or null
		if (is_null($Input))
			return true;
		elseif ($Input === "")
			return true;
		elseif (is_array($Input) && count($Input) == 0)
			return true;
		else
			return false;
	}
}

// sys_get_temp_dir()
// This function was added in PHP 5.2.1, so we need to emulate it for earlier PHP
// versions.  Support for PHP < 5.2.3 was dropped in MediaWiki 1.17 so this function
// is required until WikiDB raises its minimum requirements to MW 1.17 or above.
// (Based on http://uk.php.net/manual/en/function.sys-get-temp-dir.php, with
// various modifications.)
if (!function_exists('sys_get_temp_dir')) {
	function sys_get_temp_dir() {
		static $TempPath = false;

	// If we have already cached a valid temp path, then return it.
		if ($TempPath)
			return $TempPath;

	// Use the appropriate environment variable, if one is defined.
		$arrPossibleEnvVars = array("TMP", "TMPDIR", "TEMP");
		foreach ($arrPossibleEnvVars as $EnvVar) {
			$TempPath = getenv($EnvVar);
			if ($TempPath)
				break;
		}

	// If no environment variable is defined, use tempnam() to create a temporary
	// file and get the path element from that.
		if (!$TempPath) {
		// We use a base path that we know will be invalid on all systems (due to the
		// bad characters in it) therefore forcing PHP to use the system's temp
		// directory.
			$TempFile = tempnam(":\n\\/?><", "");

		// If the temp file exists then PHP was able to create it in the system's
		// temp folder, so we delete the file and return the path component.
			if (file_exists($TempFile)) {
				unlink($TempFile);
				$TempPath = dirname($TempFile);
			}
		}

	// If no temp file exists then no temp directory is defined.  Return False to
	// indicate an error. This should be checked for in any function that calls this!
		if (!$TempPath)
			return false;

	// Otherwise, get the real path of the temp directory, and ensure there is no
	// trailing slash, which might be added on some systems.
		$TempPath = realpath($TempPath);
		$TempPath = preg_replace('/(\\/|\\\\)$/', "", $TempPath);

		return $TempPath;
	}
}
